/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.internal.ProxyRequestInterceptor");

var encodeProxiedURIFunction = null;

/**
 * Checks to see if the incoming request contains a URL with a different origin
 * than the current page (that is either a different scheme, host, or port). If
 * the request URL has a different origin, this request will rewrite the URL to
 * go to the Jazz Web UI Server proxy.
 * 
 * @param {Object} request the request to check for different origin
 */
jazz.app.internal.ProxyRequestInterceptor = function(request){
	var url = dojo.getObject("args.url", false, request);
	if(!url) {
		return; // no url argument, early return
	}
	var urlObj = new dojo._Url(request.args.url);
	if(isDifferentOrigin(urlObj)) {
		request.args.url = encodeProxiedURIFunction(request.args.url);
		//console.warn("New proxy URL: " + request.args.url);
	}
};

/**
 * Sets the function that will be used to construct proxy URIs.
 * 
 * @param {Function} paramencodeProxiedURIFunction the function that will be used to
 *  construct proxy URIs.
 */
jazz.app.internal.ProxyRequestInterceptor.setEncodeProxiedURIFunction = function(paramEncodeProxiedURIFunction) {
	encodeProxiedURIFunction = paramEncodeProxiedURIFunction;
};

/**
 * Checks the URL object's port property and if the port property is not set,
 * sets it to either "80" (HTTP) or "443" (HTTPS).
 * 
 * @param {Object} url the URL object to check and perhaps normalize
 */
function normalizePort(url) {
	var scheme = url.scheme || "http";
	if(!url.port) {
		url.port = scheme === "http" ? "80" : "443";
	}
}

// normalize the port property of the current page for later comparison with
// incoming URLs
var LOCAL_URL = new dojo._Url(location.href); 
normalizePort(LOCAL_URL);

/**
 * Returns true if the URL object is from a different origin than the current
 * page, otherwise false.
 * 
 * @param {Object} url the URL object to check
 */
function isDifferentOrigin(url) {
	
	// optimize for the most common case (request to a non-complete URL)
	if(!url.host) {
		// if host isn't set, scheme and port don't even make sense
		return false;
	}
	
	normalizePort(url);
	
	// Compare the origin of the target URL vs. the web page's origin. If it's
	// different then proxy the request
	if(!originsAreEquivalent(url, LOCAL_URL)) {
		return true;
	}
	
	// send requests through the proxy if they are going
	// to a friend context on the same server
	if (net.jazz.ajax._friendDomains) {
		for (var i=0; i < net.jazz.ajax._friendDomains.length; i++) {
			var curFriend = new dojo._Url(net.jazz.ajax._friendDomains[i]);
			normalizePort(curFriend);
			if (originsAreEquivalent(url, curFriend) && url.path.indexOf(curFriend.path) === 0) {
				return true;
			}
		}
	} else {
		console.warn("If you are using an ajax module, please add <requiredProperty id=\"net.jazz.ajax.service.rootServicesProp\" /> to the ajax module definition");
	}
	
	
	// all properties are equal, same origin
	return false;
}

/**
 * Compare the two dojo._Url objects' scheme, host, and port. Returns 'true' if
 * all parts are the same (disregarding case differences), otherwise false.
 * 
 * @param {Object} url1
 * @param {Object} url2
 */
function originsAreEquivalent(url1, url2) {
	var props = ["scheme", "host", "port"];
	for(var i=0; i<props.length; i++) {
		var prop = props[i];
		if(url1[prop].toLowerCase() !== url2[prop].toLowerCase()) {
			return false;
		}
	}
	return true;
}

})();