/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("net.jazz.ajax.ui.internal.PageRegistry");

dojo.require("jazz.core.RegistryFactory");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("net.jazz.ajax.ui.PlatformUI");

/* 
Eclipse IPerspectiveRegistry supports:

 clonePerspective(String id, String label, IPerspectiveDescriptor desc)      Clones an existing perspective.
 IPerspectiveDescriptor 	findPerspectiveWithId(String perspectiveId)   Finds and returns the registered perspective with the given perspective id.
 IPerspectiveDescriptor 	findPerspectiveWithLabel(String label) Finds and returns the registered perspective with the given label.
 String 	getDefaultPerspective()Returns the id of the default perspective for the workbench.
 IPerspectiveDescriptor[] 	getPerspectives()  Returns a list of the perspectives known to the workbench.
 void 	revertPerspective(IPerspectiveDescriptor perspToRevert) Reverts a perspective back to its original definition as specified in the plug-in manifest.
 void 	setDefaultPerspective(String id) 

*/

var checkNotNull 	= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty 	= net.jazz.ajax.toolbox.lang.checkNotEmpty;
var RegistryFactory = jazz.core.RegistryFactory;

var removeIf 		= net.jazz.ajax.toolbox.lang.removeIf;
var findIf			= net.jazz.ajax.toolbox.lang.findIf;

var sortFunction	= null;

function getPageExtensionPoint() {
	var extensionPoint = RegistryFactory.getRegistry().getExtensionPoint("net.jazz.ajax.pages");
	
	if(!extensionPoint) {
		throw new Error("Page extension point not found");
	}
	
	return extensionPoint;
}

function getApplicationPagesExtensionPoint() {
	var extensionPoint = RegistryFactory.getRegistry().getExtensionPoint("net.jazz.ajax.applicationPageBindings");
	
	if(!extensionPoint) {
		throw new Error("Application Pages Binding extension point not found");
	}
	
	return extensionPoint;
}

function getPage(extension, configElement) {
	checkNotNull(extension, "extension");
	checkNotNull(configElement, "configElement");
	
	checkNotNull(extension.getNamespace(), "extension namespace");
	checkNotNull(configElement.getAttribute("id"), "page id");
	checkNotNull(configElement.getAttribute("name"), "page name");
	
	//dojo.lang.assert(configElement.getName() == "page", "invalid configuration element: " + configElement.getName());

	var page = {
		id: configElement.getAttribute("id"),
		name: configElement.getAttribute("name"),
		href: configElement.getAttribute("href"),
		widget: configElement.getAttribute("widget"),
		defaultAction: configElement.getAttribute("defaultAction"),
		removalPropertyId: configElement.getAttribute("removalPropertyId"),
		component: extension.getNamespace(),
		getUri: function() { return "#action=jazz.viewPage&id=" + this.id; },
		actions: [],
		cssUri: configElement.getAttribute("cssUri"),
		isDefault: false
	};
	
	dojo.forEach(configElement.getChildren(), function(childCfgElement) {
		if (configElement.getName() !== "action") {
			return;
		}
		checkNotNull(childCfgElement.id);
		getPage.actions.push({});
	});

	var applicationCSSUri = net.jazz.ajax.ui.PlatformUI._getCssUri();
	if (applicationCSSUri === null && page.cssUri !== null) {
		console.warn("Application ["+net.jazz.ajax.ui.PlatformUI._getApplicationId()+"] has no cssUri set but page ["+page.id+"] does. Setting page cssUri to null");
		page.cssUri = null;
	}	
	return page;
}

function findPageWithId(id) {
	checkNotNull(id, "id");
	
	var matchedPage = null;
	dojo.forEach( getPageExtensionPoint().getExtensions(), function(extension) {
		dojo.forEach(extension.getConfigurationElements(), function(configElement) {
			if (configElement.getAttribute("id") == id) {
				matchedPage = getPage(extension, configElement);	
			}
		});	
	});	
	return matchedPage;
}

net.jazz.ajax.ui.internal.PageRegistry.findPageWithId = findPageWithId;

function getPages() {
	var workbenchId  = net.jazz.ajax.ui.PlatformUI._getApplicationId();
	var pageArray = [];

	dojo.forEach( getApplicationPagesExtensionPoint().getExtensions(), function(extension) {
		dojo.forEach(extension.getConfigurationElements(), function(configElement) {
			if (configElement.getName() !== "application") { return; }
			var applicationId = configElement.getAttribute("id");
			if (applicationId == workbenchId) {
				var defaultFound = false;
				dojo.forEach(configElement.getChildren(), function(childCfgElement) {
					if (childCfgElement.getName() !== "page") { return; }
					var page = findPageWithId(childCfgElement.getAttribute("id"));
					if (page !== null) {
						var isDefault = childCfgElement.getAttribute("default");
						if (isDefault !== null && isDefault === "true") {
							if (defaultFound === true) {
								console.error("A default page has already been set. Page with an id of ["+childCfgElement.getAttribute("id")+"] will not be set as the default");	
							}
							else {
								defaultFound = true;	
								page.isDefault = true;							
							}
						}
						var pushPage = true;
						if (page.removalPropertyId !== null) {
							var removalProperty = dojo.getObject(page.removalPropertyId);
							if (removalProperty && removalProperty === true) {
								pushPage = false;
							}
						}
						if (pushPage) {
							pageArray.push(page);
						}
					}
				});					
			}
		});		
	});
	
	if(sortFunction){
		pageArray.sort(sortFunction);
	}
	
	return pageArray;
}

net.jazz.ajax.ui.internal.PageRegistry.getPages = getPages;

function getDefaultPage() {
	var pages = getPages();
	for (var i = 0; i < pages.length; i++) {
		if (pages[i].isDefault === true) {
			return pages[i];
		}
	}
	return pages[0];
}
net.jazz.ajax.ui.internal.PageRegistry.getDefaultPage = getDefaultPage;

function setSortFunction(/*Function*/sortFunctionArg){
	if(!dojo.isFunction(sortFunctionArg)){
		throw new Error("PageRegistry#setSortFunction: argument is not a function");
	}
	sortFunction = sortFunctionArg;
}

net.jazz.ajax.ui.internal.PageRegistry.setSortFunction = setSortFunction;
})();