/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ActiveServicesView");

dojo.require("dijit._Widget");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util")

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ActiveServicesViewMessages");

dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");

var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var formatTimeFromMilliseconds = com.ibm.team.repository.web.ui.internal.admin.util.formatTimeFromMilliseconds;
var replaceNewLineWithPaddedBreaks = com.ibm.team.repository.web.ui.internal.admin.util.replaceNewLineWithPaddedBreaks;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ActiveServicesView", [dijit._Widget, dijit._Templated], {

	templateString: "<div class=\"com-ibm-team-repository-active\"><div dojoAttachPoint=\"headerArea\"></div><div dojoAttachPoint=\"contentArea\"></div></div>",
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ActiveServicesViewMessages");
	},	
	postCreate: function() {
		this.header = new WizardStepHeader({ 
			stepTitle: this.messages.heading,
			reset: dojo.hitch(this,"refresh")
		});
		this.headerArea.appendChild(this.header.domNode);
		this._fetchAndDisplayServiceInfo();
	},
	
	refresh: function () {
		this.header.clearMessages();
		this._fetchAndDisplayServiceInfo();
	},
	
	_fetchAndDisplayServiceInfo: function() {
		var srh= new ServiceResponseHandler(this, "_displayActiveServicesInfo", "displayServiceCallbackErrorMessage");
		ServerStatusClient.getActiveServiceInfo(srh)
	},


	displayServiceCallbackErrorMessage: function(errorObj) {
		this.header.displayServiceCallbackErrorMessage(errorObj);
	},

	_displayActiveServicesInfo: function(activeServiceInfo) {
		dojox.data.dom.removeChildren(this.contentArea);
		var servicesDiv = document.createElement("div");	
		var services = activeServiceInfo.services;
		//KMB TODO: sort service list
		if (services) {
			for (var i=0; i<services.length; i++) {
				this._appendActiveServiceSection(services[i], servicesDiv, activeServiceInfo.currentTime);
			}
		}
		this.contentArea.appendChild(servicesDiv);
		this.header.showTimestamp();
	},
	
	_appendActiveServiceSection: function(service, node, currentTime) {
		// TODO make this a simple template widget defined in this file
		var elapsedTime = currentTime - service.startTime;
		elapsedTime = formatTimeFromMilliseconds(elapsedTime);
		elapsedTime = dojo.string.substitute(this.messages.runningTime, [ elapsedTime ]);
		
		var div = document.createElement("div");
		div.className = "right-area";
		
		var timeSpan = document.createElement("span");
		timeSpan.className = "running-time";
		timeSpan.innerHTML = elapsedTime;
		
		var requesterSpan = document.createElement("span");
		requesterSpan.className = this.messages.requester;
		requesterSpan.innerHTML = this.messages.by + service.userId;
		
		var serviceNameSpan = document.createElement("div");
		serviceNameSpan.className = "service-name";
		serviceNameSpan.innerHTML = service.service + "." + service.method;
		
		var serviceTitleDiv =  document.createElement("div");
		serviceTitleDiv.className = "service-title";
		
		div.appendChild(requesterSpan);
		div.appendChild(timeSpan); 
		serviceTitleDiv.appendChild(serviceNameSpan); 
		serviceTitleDiv.appendChild(div);
		
		var traceDiv = document.createElement("div");
		traceDiv.className = "service-trace";
		traceDiv.innerHTML = replaceNewLineWithPaddedBreaks(service.stacktrace, true);
		
		serviceTitleDiv._traceDiv = traceDiv;
		serviceTitleDiv._traceDiv.style.display = "none";
		
		serviceTitleDiv.onclick = function(){
			newDisplay = serviceTitleDiv._traceDiv.style.display === "block"
				? "none" : "block";
			serviceTitleDiv._traceDiv.style.display = newDisplay;
		}
		
		var activeService = document.createElement("div");
		activeService.className = "active-service";
		activeService.appendChild(serviceTitleDiv);
		activeService.appendChild(traceDiv);
	
		node.appendChild(activeService);
	}
});

})();