/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("net.jazz.ajax.service.ui.userdata.util");

dojo.require("dojo.cookie");
dojo.require("jazz.client.xhr");

var COOKIE_USER_ID = "net-jazz-ajax-cookie-userName";
var util = net.jazz.ajax.service.ui.userdata.util;
var DEFAULT_SERVER_ALIAS = "jrs1";

util.getFavorites = function (searchTerm) {
	searchTerm = searchTerm || "";
	if (util.getLoggedInUser()) {
		return jazz.client.xhrGet({
			url: "/_userdata/" + util.getLoggedInUser() + "/favorite",
			content: {
				searchTerm: searchTerm
			},
			handleAs: "json",
			headers: {
				accept: "text/json"
			}
		});
	}
	else {
		return null;
	}
};

util.getFavoriteItem = function (uri) {
	if (util.getLoggedInUser()) {
		return jazz.client.xhrGet({
			url: "/_userdata/" + util.getLoggedInUser() + "/favoriteItem",
			content: {
				uri: uri
			},
			handleAs: "json",
			headers: {
				accept: "text/json"
			}
		});
	} else {
		return null;
	}
};

util.getFavoriteFolderItem = function (folder) {
	if (util.getLoggedInUser()) {
		return jazz.client.xhrGet({
			url: "/_userdata/" + util.getLoggedInUser() + "/favoriteFolderItem",
			content: {
				folder: folder
			},
			handleAs: "json",
			headers: {
				accept: "text/json"
			}
		});
	} else {
		return null;
	}
};

util.getHistory = function (searchTerm) {
	searchTerm = searchTerm || "";
	if (util.getLoggedInUser()) {
		return jazz.client.xhrGet({
			url: "/_userdata/" + util.getLoggedInUser() + "/history",
			content: {
				searchTerm: searchTerm
			},
			handleAs: "json",
			headers: {
				accept: "text/json"
			}
		});
	} else {
		return null;
	}
};

util.getHistoryItem = function (uri) {
	if (util.getLoggedInUser()) {
		return jazz.client.xhrGet({
			url: "/_userdata/" + util.getLoggedInUser() + "/historyItem",
			content: {
				uri: uri
			},
			handleAs: "json",
			headers: {
				accept: "text/json"
			}
		})
	} else {
		return null;
	}
}

util.addHistory = function (data) {
	if (util.getLoggedInUser() && data && data.uri) {
		var tagArray = [];
		if (data.tags) {
			tagArray = data.tags.split(/\s*,\s*/);
		}
		return jazz.client.xhrPost({
			url: "/_userdata/" + util.getLoggedInUser() + "/history",
			content: {
				uri: data.uri,
				title: data.title || data.uri,
				comments: data.comments || "",
				tag: tagArray
			}
		});
	}
	else {
		return null;
	}
};

util.addFavorite = function (data) {
	if (util.getLoggedInUser() && data && data.uri) {
		var tagArray = [];
		if (data.tags) {
			tagArray = data.tags.split(/\s*,\s*/);
		}
		return jazz.client.xhrPost({
			url: "/_userdata/" + util.getLoggedInUser() + "/favorite",
			content: {
				uri: data.uri,
				title: data.title || data.uri,
				comments: data.comments || "",
				tag: tagArray,
				folder: data.folder,
				sortKey: data.sortKey
			}
		});
	}
	else {
		return null;
	}
};

util.removeHistory = function (paths) {
	if (util.getLoggedInUser()) {
		return jazz.client.xhrDelete({
			url: "/_userdata/" + util.getLoggedInUser() + "/history",
			content: {
				uri: paths
			}
		});
	}
}

util.removeFavorites = function (paths) {
	if (util.getLoggedInUser()) {
		return jazz.client.xhrDelete({
			url: "/_userdata/" + util.getLoggedInUser() + "/favorite",
			content: {
				uri: paths
			}
		});
	}
}

util.isFavorite = function (path) {
	if (util.getLoggedInUser() && path) {
		return jazz.client.xhrGet({
			url: "/_userdata/" + util.getLoggedInUser() + "/isFavorite",
			handleAs: "json",
			headers: {
				accept: "text/json"
			},
			content: {
				uri: path
			}
		});
	}
	else {
		return null;
	}
}

util.getLoggedInUser = function () {
	return dojo.cookie(COOKIE_USER_ID);
};

})();
